/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifdef HAVE_CONFIG_H
#include "autoconfig.h"
#endif

#include <stdio.h>

#include "webserver.h"
#include "exceptions.h"
#include "util.h"
#include "json_device_web_handler.h"
#include "json_dsm_web_handler.h"
#include "json_hardware_web_handler.h"
#include "json_task_web_handler.h"
#include "html_report_web_handler.h"
#include "config_manager.h"
#include "log.h"

namespace dSA
{

Webserver::Webserver(int port, std::string docroot) : m_document_root(docroot)
{
    m_port = util::str(port);

    ConfigManager* cfg = ConfigManager::getInstance();
    std::string interface = cfg->getOption(CFG_WEBSERVER_INTERFACE);
    if (interface.empty())
    {
        interface = "0.0.0.0";
    }
    else
    {
        interface = netutil::if_to_ip(interface);
    }

    interface = interface + ":" + m_port;

    const char *options[] =
    {
      "document_root", m_document_root.c_str(),
      "listening_ports", interface.c_str(),
      "num_threads", "4",
      NULL
    };

    m_context = mg_start(&mongoose_callback, options);
    if (m_context == NULL)
    {
        throw _Exception("Could not start webserver");
    }

    FILE_LOG(logINFO) << "Webserver listening on " << interface;

}

void *Webserver::mongoose_callback(enum mg_event event,
                                  struct mg_connection* connection,
                                  const struct mg_request_info* request_info)
{
    if (event != MG_NEW_REQUEST)
    {
        return NULL;
    }

    try
    {
        std::string uri = request_info->uri;
        if (uri.find("/dsa/json/task") == 0)
        {
            JSONTaskWebHandler jsw(connection, request_info);
            jsw.process();
            return connection;
        }
        else if (uri.find("/dsa/json/device") == 0)
        {
            JSONdeviceWebHandler jdsdw(connection, request_info);
            jdsdw.process();
            return connection;
        }
        else if (uri.find("/dsa/json/dsm") == 0)
        {
            JSONdSMWebHandler jdsmw(connection, request_info);
            jdsmw.process();
            return connection;
        }
        else if (uri.find("/dsa/json/hardware") == 0)
        {
            JSONHardwareWebHandler jhw(connection, request_info);
            jhw.process();
            return connection;
        }
        else if (uri.find("/dsa/html/report") == 0)
        {
            HTMLReportWebHandler html(connection, request_info);
            html.process();
            return connection;
        }
    }
    catch (Exception ex)
    {
        printf("TODO: return code, Exception caught in webserver\n");
    }
    catch (...)
    {
        printf("TODO: return code, Exception caught in webserver\n");
    }

    return NULL;
}

Webserver::~Webserver()
{
    if (m_context != NULL)
    {
        mg_stop(m_context);
    }
}

}; // namespace


